@@ -3,7 +3,7 @@ from collections import Counter, defaultdict
 from copy import copy
 from numbers import Integral
 from itertools import chain
-from typing import Union, Optional, List
+from typing import Union, Optional, List, Tuple
 
 import nltk
 import numpy as np
@@ -12,6 +12,10 @@ from gensim import corpora
 
 from Orange.data import ContinuousVariable, DiscreteVariable, \
     Domain, RowInstance, Table, StringVariable
+# uncomment when Orange3==3.27 is available
+# from Orange.data.util import get_unique_names
+# remove when Orange3==3.27 is available
+from orangecontrib.text.vectorization.base import get_unique_names
 from orangecontrib.text.vectorization import BowVectorizer
 
 
@@ -67,8 +71,10 @@ class Corpus(Table):
         self.ngram_range = (1, 1)
         self.attributes = {}
         self.pos_tags = None
-        self.used_preprocessor = None   # required for compute values
+        from orangecontrib.text.preprocess import PreprocessorList
+        self.__used_preprocessor = PreprocessorList([])   # required for compute values
         self._titles: Optional[np.ndarray] = None
+        self._pp_documents = None  # preprocessed documents
 
         if domain is not None and text_features is None:
             self._infer_text_features()
@@ -81,6 +87,21 @@ class Corpus(Table):
             Table._init_ids(self)
         self._set_unique_titles()
 
+    @property
+    def used_preprocessor(self):
+        return self.__used_preprocessor  # type: PreprocessorList
+
+    @used_preprocessor.setter
+    def used_preprocessor(self, pp):
+        from orangecontrib.text.preprocess import PreprocessorList, Preprocessor
+
+        if isinstance(pp, PreprocessorList):
+            self.__used_preprocessor = PreprocessorList(list(pp.preprocessors))
+        elif isinstance(pp, Preprocessor):
+            self.__used_preprocessor.preprocessors.append(pp)
+        else:
+            raise NotImplementedError
+
     def set_text_features(self, feats):
         """
         Select which meta-attributes to include when mining text.
@@ -207,8 +228,10 @@ class Corpus(Table):
         self._tokens = None     # invalidate tokens
         self._set_unique_titles()
 
-    def extend_attributes(self, X, feature_names, feature_values=None,
-                          compute_values=None, var_attrs=None, sparse=False):
+    def extend_attributes(
+            self, X, feature_names, feature_values=None, compute_values=None,
+            var_attrs=None, sparse=False, rename_existing=False
+        ):
         """
         Append features to corpus. If `feature_values` argument is present,
         features will be Discrete else Continuous.
@@ -220,20 +243,51 @@ class Corpus(Table):
             compute_values (list): Compute values for corresponding features.
             var_attrs (dict): Additional attributes appended to variable.attributes.
             sparse (bool): Whether the features should be marked as sparse.
+            rename_existing (bool): When true and names are not unique rename
+                exiting features; if false rename new features
         """
-        if self.X.size == 0:
-            self.X = X
-        elif sp.issparse(self.X) or sp.issparse(X):
-            self.X = sp.hstack((self.X, X)).tocsr()
+        def _rename_features(additional_names: List) -> Tuple[List, List, List]:
+            cur_attr = list(self.domain.attributes)
+            cur_class = self.domain.class_var
+            cur_meta = list(self.domain.metas)
+            if rename_existing:
+                current_vars = (
+                        cur_attr + (
+                    [cur_class] if cur_class else []) + cur_meta
+                )
+                current_names = [a.name for a in current_vars]
+                new_names = get_unique_names(
+                    additional_names, current_names, equal_numbers=False
+                )
+                renamed_vars = [
+                    var.renamed(n) for var, n in zip(current_vars, new_names)
+                ]
+                cur_attr = renamed_vars[:len(cur_attr)]
+                cur_class = renamed_vars[len(cur_attr)] if cur_class else None
+                cur_meta = renamed_vars[-len(cur_meta):]
+            return cur_attr, cur_class, cur_meta
+
+        if sp.issparse(self.X) or sp.issparse(X):
+            X = sp.hstack((self.X, X)).tocsr()
         else:
-            self.X = np.hstack((self.X, X))
+            X = np.hstack((self.X, X))
 
         if compute_values is None:
             compute_values = [None] * X.shape[1]
         if feature_values is None:
             feature_values = [None] * X.shape[1]
 
-        new_attr = self.domain.attributes
+        # rename existing variables if required
+        curr_attributes, curr_class_var, curr_metas = _rename_features(
+            feature_names
+        )
+        if not rename_existing:
+            # rename new feature names if required
+            feature_names = get_unique_names(
+                self.domain, feature_names, equal_numbers=False
+            )
+
+        additional_attributes = []
         for f, values, cv in zip(feature_names, feature_values, compute_values):
             if values is not None:
                 var = DiscreteVariable(f, values=values, compute_value=cv)
@@ -244,28 +298,42 @@ class Corpus(Table):
                 cv.variable = var
             if isinstance(var_attrs, dict):
                 var.attributes.update(var_attrs)
-            new_attr += (var, )
+            additional_attributes.append(var)
 
         new_domain = Domain(
-                attributes=new_attr,
-                class_vars=self.domain.class_vars,
-                metas=self.domain.metas
+                attributes=curr_attributes + additional_attributes,
+                class_vars=curr_class_var,
+                metas=curr_metas
+        )
+        return Corpus(
+            new_domain,
+            X,
+            self.Y.copy(),
+            self.metas.copy(),
+            self.W.copy(),
+            copy(self.text_features)
         )
-        self.domain = new_domain
 
     @property
     def documents(self):
-        """
-        Returns: a list of strings representing documents — created by joining
-            selected text features.
-        """
+        """ Returns a list of strings representing documents — created
+        by joining selected text features. """
         return self.documents_from_features(self.text_features)
 
+    @property
+    def pp_documents(self):
+        """ Preprocessed documents (transformed). """
+        return self._pp_documents or self.documents
+
+    @pp_documents.setter
+    def pp_documents(self, documents):
+        self._pp_documents = documents
+
     @property
     def titles(self):
         """ Returns a list of titles. """
         assert self._titles is not None
-        return self._titles.tolist()
+        return self._titles
 
     def documents_from_features(self, feats):
         """
@@ -298,19 +366,25 @@ class Corpus(Table):
     def tokens(self):
         """
         np.ndarray: A list of lists containing tokens. If tokens are not yet
-        present, run default preprocessor and save tokens.
+        present, run default preprocessor and return tokens.
         """
         if self._tokens is None:
-            self._apply_base_preprocessor()
+            return self._base_tokens()[0]
         return self._tokens
 
     def has_tokens(self):
         """ Return whether corpus is preprocessed or not. """
         return self._tokens is not None
 
-    def _apply_base_preprocessor(self):
-        from orangecontrib.text.preprocess import base_preprocessor
-        base_preprocessor(self)
+    def _base_tokens(self):
+        from orangecontrib.text.preprocess import BASE_TRANSFORMER, \
+            BASE_TOKENIZER, PreprocessorList
+
+        # don't use anything that requires NLTK data to assure async download
+        base_preprocessors = PreprocessorList([BASE_TRANSFORMER,
+                                               BASE_TOKENIZER])
+        corpus = base_preprocessors(self)
+        return corpus.tokens, corpus.dictionary
 
     @property
     def dictionary(self):
@@ -318,7 +392,7 @@ class Corpus(Table):
         corpora.Dictionary: A token to id mapper.
         """
         if self._dictionary is None:
-            self._apply_base_preprocessor()
+            return self._base_tokens()[1]
         return self._dictionary
 
     def ngrams_iterator(self, join_with=' ', include_postags=False):
@@ -369,6 +443,7 @@ class Corpus(Table):
         c.name = self.name
         c.used_preprocessor = self.used_preprocessor
         c._titles = self._titles
+        c._pp_documents = self._pp_documents
         return c
 
     @staticmethod
@@ -481,8 +556,8 @@ class Corpus(Table):
                     new._tokens = np.array([orig._tokens[key]])
                     new.pos_tags = None if orig.pos_tags is None else np.array(
                         [orig.pos_tags[key]])
-                elif isinstance(key, list) or isinstance(key, np.ndarray) or isinstance(key,
-                                                                                        slice):
+                elif isinstance(key, list) or isinstance(key, np.ndarray) \
+                        or isinstance(key, slice) or isinstance(key, range):
                     new._tokens = orig._tokens[key]
                     new.pos_tags = None if orig.pos_tags is None else orig.pos_tags[key]
                 elif key is Ellipsis:
